
package com.example.sub2;

import java.lang.reflect.Method;
import java.util.Random;

import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

public aspect MyAspect {

    private final Logger log = LoggerFactory.getLogger(this.getClass());

    // Ensure that this aspect is woven around the AnnotationTransactionAspect
    declare precedence : MyAspect,org.springframework.transaction.aspectj.*;

    /*
     * Attempt to execute a transactional method. If the method throws any exception which is translated
     * to a subclass of TransientDataAccessException, the method will be executed again, with a delay
     * between attempts.
     */
    Object around(Object txObject, Transactional annotation) : transactionalMethodExecution(txObject, annotation) {
        this.log.info("around: " + this.getClass());
        return proceed(txObject, annotation);
    }

    /*
     * Matches the execution of any public method in a type with the @Transactional annotation, or any subtype of a type with the
     * @Transactional annotation.
     */
    private pointcut executionOfPublicMethodInAnnotatedType(Transactional annotation) :
      execution(public * ((@Transactional *)+).*(..)) && within(@Transactional *)
        && annotationPropagationRequiresTransaction(annotation);

    /*
     * Matches the execution of any method with the @Transactional annotation.
     */
    private pointcut executionOfAnnotatedMethod(Transactional annotation) :
      execution(@Transactional * *(..)) && annotationPropagationRequiresTransaction(annotation);

    /*
     * Matches any @Transactional annotation with a propagation behavior of
     * either Propagation.REQUIRES or Propagation.REQUIRES_NEW
     */
    private pointcut annotationPropagationRequiresTransaction(Transactional annotation) :
      @annotation(annotation)
        && if(annotation.propagation() == Propagation.REQUIRED || annotation.propagation() == Propagation.REQUIRES_NEW);

    /*
     * Main pointcut - matched join points will have Spring transaction management applied.
     */
    private pointcut transactionalMethodExecution(Object txObject, Transactional annotation) :
      (executionOfPublicMethodInAnnotatedType(annotation) || executionOfAnnotatedMethod(annotation))
        && this(txObject);
}

