#!/bin/sh
# $Id: verifyweave.sh 2527 2013-04-12 17:11:55Z archie $

set -e

verify_weave()
{
    ANAME=`echo "${1}" | sed 's|^.*\.\([^.]\{1,\}\)$|@\1|g'`
    ACLASS=`echo "${1}" | sed 's|\.|/|g'`
    ASPECT="$2"

    echo "Verifying ${ANAME} classes got woven with ${ASPECT}..." 1>&2
    (
        cd build/classes
        find . -name '*.class' -a \! -name '*$AjcClosure.class' -print \
          | xargs grep -lF "${ACLASS}" \
          | sed 's|^\./\(.*\)\.class$|\1|g' \
          | grep -vEx 'com/example/sub./MyAspect' \
          | while read CLASS; do
            DIR=`dirname "${CLASS}"`
            SNAME=`basename "${CLASS}"`
            CLASS_FILES=`find "${DIR}" -name "${SNAME}"'*.class' -print | sed 's|\.class$||g'`
            if ! javap -c -private -classpath . ${CLASS_FILES} | grep -qF "${ASPECT}.aspectOf"; then
                echo "Error: ${CLASS} contains ${ANAME} but did not get woven with ${ASPECT}" 1>&2
                touch verify_weave.error
            fi
        done
    )
}

rm -f verify_weave.error
verify_weave org.springframework.transaction.annotation.Transactional   AnnotationTransactionAspect
verify_weave org.springframework.transaction.annotation.Transactional   MyAspect
verify_weave org.springframework.beans.factory.annotation.Configurable  AnnotationBeanConfigurerAspect
verify_weave org.dellroad.stuff.vaadin7.VaadinConfigurable              VaadinConfigurableAspect

if [ -e verify_weave.error ]; then
    exit 1
fi

